@@ -2,8 +2,15 @@ use Config;
 use Glib::Object::Introspection;
 use Test::More;
 
-unless (-e qq(build/libregress.$Config{dlext}) &&
-        -e qq(build/libgimarshallingtests.$Config{dlext}))
+my $lib_ext;
+if ( $^O =~ /darwin/ ) {
+    $lib_ext = $Config{so};
+} else {
+    $lib_ext = $Config{dlext};
+}
+
+unless (-e qq(build/libregress.$lib_ext) &&
+        -e qq(build/libgimarshallingtests.$lib_ext))
 {
   plan skip_all => 'Need the test libraries';
 }
@@ -43,7 +50,8 @@ sub delta_ok ($$;$) {
 
 sub check_gi_version {
   my ($x, $y, $z) = @_;
-  return !system ('pkg-config', "--atleast-version=$x.$y.$z", 'gobject-introspection-1.0');
+  #return !system ('pkg-config', "--atleast-version=$x.$y.$z", 'gobject-introspection-1.0');
+  return Glib::Object::Introspection->CHECK_VERSION ($x, $y, $z);
 }
 
 1;
